/*******************************************************************

    YAPP(tm) protocol for Packet Radio binary file transfer

	(c) 1986, Jeff Jacobsen - WA7MBL

	Revision 1.1   06/23/86

	----------------------------------------------------------

	IW3FQG's Yapp Definition

	This source include resume on download

    (c) Copyright 1990/1993 Marco Savegnago IW3FQG

*******************************************************************/

#ifndef __TYAPP_H
#define __TYAPP_H

// PKLib	include

#include <xferprot/tbinxfer.h>

//	CRTL include

#include <dir.h>
#include <io.h>

const byte	NIL 	= '\x00';
const byte	SOH 	= '\x01';
const byte	STX 	= '\x02';
const byte	ETX 	= '\x03';
const byte	EOT 	= '\x04';
const byte	ENQ 	= '\x05';
const byte	ACK 	= '\x06';
const byte	DLE 	= '\x10';
const byte	NAK 	= '\x15';
const byte	CAN 	= '\x18';

const maxRetry		= 6;
const maxYappFrame 	= 256;

typedef char yappTStr[3];

struct 	yappTFrame
{
	byte cmd;
	byte count;
	byte frame[maxYappFrame];

	yappTFrame() : cmd(STX), count(0) {}
};

typedef byte yappFrameBuffer[maxYappFrame];


class TYapp : public TBinaryXferProtcol
{
	enum yapp_states
	{//  *	 *                  *           *       *   *
		SN, SR, SH, SD, SE, ST, RC, RH, RD, AB, CW, CR, SA
	} state ;

	enum packet_type
	{
		UK, RR, RF, SI, HD, DT, EF, ET, NR, CN, CA, RI, TX, UU, TM, AF, AT
	} ptype ;

	static yappTStr strRcvReady;	// RR 	{ ACK, 0x01 }
	static yappTStr strRcvFile;		// RF	{ ACK, 0x02 }
	static yappTStr strAckEOF;		// AF   { ACK, 0x03 }  [Yapp 1.1]
	static yappTStr strAckEOT;		// AT	{ ACK, 0x04 }  [Yapp 1.1]
	static yappTStr strSendInit;	// SI	{ ENQ, 0x01 }

	static yappTStr strSendEOF;		// EF	{ ETX, 0x01 }
	static yappTStr strSendEOT;		// ET	{ EOT, 0x01 }
	static yappTStr strCanACK;		// CA	{ ACK, 0x05 }
	static yappTStr strNotReady;	// NR   { NAK, 0x00 }
    static yappTStr strCancel;		// CN   { CAN, 0x00 }

	yappFrameBuffer frameBuffer;

	word 			frameLen;

	int 	sendInitCount;

	char*	theCurrentFilename;

	char 	yappHeaderFileName[MAXPATH];
	long 	yappHeaderFileLenght;
	struct 	ftime yappHeaderFiletime;

    Boolean	yappResume;

	Boolean extendedHeader;

	Boolean waitReady();

	void getPacket();
    void getPacketString();
	Boolean splitHeader();
	byte makeHeader();

	void dispPacket() {}

	void Dispatch(yapp_states state);

#ifdef __USEDDVT

	virtual void sendInit() 		= [ SN ];
	virtual void sendInitRetry() 	= [ SR ];
	virtual void sendHdr() 			= [ SH ];
    virtual void sendData() 		= [ SD ];
    virtual void sendEOF() 			= [ SE ];
    virtual void sendEOT() 			= [ ST ];
    virtual void receive() 			= [ RC ];
    virtual void recvHdr() 			= [ RH ];
	virtual void recvData() 		= [ RD ];
	virtual void cancel() 			= [ AB ];
	virtual void canWait() 			= [ CW ];
    virtual void canRecd() 			= [ CR ];

#elif defined (__YAPPUSESWITCH) || defined (__YAPPUSEFPTR)

	void sendInit();
	void sendInitRetry();
	void sendHdr();
	void sendData();
	void sendEOF();
	void sendEOT();
	void receive();
	void recvHdr();
	void recvData();
	void cancel();
	void canWait();
	void canRecd();

#endif

#if defined (__YAPPUSEFPTR)

	typedef void (TYapp::*conFP )();

	static 	conFP	stateDispatchTable[];

#endif

public:

	Boolean xfer();

	TYapp(TStream *port, fstream *afstream, xferType type, char *filename, Boolean resume=False);

	~TYapp();
};

#if defined (__YAPPUSEFPTR)
inline void TYapp::Dispatch( yapp_states state )
{
	stateDispatchTable[state];
}
#endif

// DDVT	helper functions

typedef void ( TYapp::*ConFP )();
typedef void ( *FP)();

/* ---------------------------------------------------------- *\
| _DDVTdispathNULL is a function in the RTL.  There are two    |
| versions,one for far vtables, one for near.  They both return|
| pointers to pointers to the address of the DDVT function, or |
| NULL if one is not found..                                   |
\* ---------------------------------------------------------- */

#ifdef FARVTBL
	#define GetVptr(thisPtr)    (*(void far **)(thisPtr))
	FP far * _DDVTdispatchNULL(void far *, int);
#else
	#define GetVptr( thisPtr)   (*(void near **)(thisPtr))
	FP far * _DDVTdispatchNULL(void near *, int);
#endif

/* ----------------------------------------------------------- *\
|  NOTE: We are assuming the vtable pointer is at the very top  |
|  of the class.  See the 'Borland Open Architecture' Manual    |
|  regarding the scenarios where this assumption is not valid.  |
\* ----------------------------------------------------------- */

/*************************************************************************

  For more information regarding the DDVTs and their layouts ( etc.
  etc. ) see the "Borland Open Architecture" HandBook or Borland TI1026.

*************************************************************************/

#endif // !#define __TYAPP_H
